/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Calculates what portion of a digit string is the country code
 *
 * Copyright (C) 1999, Mark Spencer
 * Copyright (C) 2003, Jefferson Noxon
 * Copyright (C) 2005, Vikram Rangnekar 
 *
 * Mark Spencer <markster@linux-support.net>
 * Jefferson Noxon <jeff@debian.org>
 * Vikram Rangnekar <vicram@gmail.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <sys/types.h>
#include <asterisk/options.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/lock.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

static char *tdesc = "Calculates what portion of a digit string is the country code";

static char *g_descrip =
	"  GetCC(varname=${EXTEN}): Seta a variable to the country code portion of the extension\n"
	"Calculates what portion of a digit string is the country code using\n"
	"known country code length patterns used throughout the world.  Can\n"
	"analyze number without a database hit\n\n"
	"The input 'pattern' to the routine MUST NOT contain the leading\n"
	"International Access digits for the country you are in.  (EXAMPLE:  Do not\n"
	"have the leading '011' if in the US, or '00/01' in other countries)\n\n"
	"watch how the routine, when given something like 52583839337737 will find\n"
	"and spit back 52 as the country code\n";


static char *g_app = "GetCC";

static char *g_synopsis = "sets a variable to the country code portion of the extension";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int findCCodeInPatt(char *buff, char *pattern)
{
	int   numdigs=2; /* Set default length to 2 digits */
	int   d1=0;
	int   d2=0;

	d1 = pattern[0]-'0';
	d2 = pattern[1]-'0';
	buff[0]='\0';

	switch (d1) {
	case 1:
	case 7:
		numdigs=1;
		break;
	case 2:
		if (d2!=0 && d2!=7) numdigs=3;
		break;
	case 3:
		if (d2==5 || d2==7 || d2==8) numdigs=3;
		break;
	case 4:
		if (d2==2) numdigs=3;
		break;
	case 5:
		if (d2==0 || d2==9) numdigs=3;
		break;
	case 6:
		if (d2 >= 7) numdigs=3;
		break;
	case 8:
		if (d2==0 || d2==3 || d2==5 || d2>=7) numdigs=3;
		break;
	case 9:
		if (d2==6 || d2==7 || d2==9) numdigs=3;
		break;
	default:
		numdigs=0;
	} /* end switch (d1) */

	if (d2<0 || d2>9) {
		/* No good if second digit is not a digit */
		numdigs=0;
	}
	else {
		if (strlen(pattern) < numdigs) {
			/* No good if pattern not long enough */
			numdigs=0;
		}
		else {
			/* Copy country code portion of pattern into buff */
			strncpy(buff,pattern,numdigs);
			buff[numdigs]='\0';
	 	}
	}
	return(numdigs);
}

static int getcc_exec (struct ast_channel *chan, void *data)
{
	int arglen;
	char *argv, *varname,  *key;
	char buff[5];

	arglen = strlen (data);
	argv = alloca (arglen + 1);
	if (!argv) {	/* Why would this fail? */
		ast_log (LOG_DEBUG, "Memory allocation failed\n");
		return 0;
	}
	memcpy (argv, data, arglen + 1);

	if (strchr (argv, '=')) {
		varname = strsep (&argv, "=");
		key = strsep (&argv, "\0");
		if (!varname || !key) {
			ast_log (LOG_DEBUG, "Ignoring; Syntax error in argument\n");
			return 0;
		}
		if (option_verbose > 2)
			ast_verbose (VERBOSE_PREFIX_3 "GetCC: varname=%s, pattern=%s\n", varname, key);
		if (findCCodeInPatt(buff, key) > 0) {
			pbx_builtin_setvar_helper (chan, varname, buff);
			if (option_verbose > 2)
				ast_verbose (VERBOSE_PREFIX_3 "GetCC: set variable %s to %s\n", varname, key);
		} else {
			if (option_verbose > 2)
			ast_verbose (VERBOSE_PREFIX_3 "GetCC: Bad number pattern '%s'. Make sure the IAD is stripped.\n", key);
			/* Send the call to n+101 priority, where n is the current priority */
			if (ast_exists_extension (chan, chan->context, chan->exten, chan->priority + 101, chan->callerid))
				chan->priority += 100;
		}

	} else {
		ast_log (LOG_DEBUG, "Ignoring, no parameters\n");
	}
	return 0;
}


int unload_module (void)
{
	int retval = 0;

	STANDARD_HANGUP_LOCALUSERS;
	retval |= ast_unregister_application (g_app);

	return retval;
}

int load_module (void)
{
	int retval;

	retval = ast_register_application (g_app, getcc_exec, g_synopsis, g_descrip);
	return retval;
}

char *description (void)
{
	return tdesc;
}

int usecount (void)
{
	int res;
	STANDARD_USECOUNT (res);
	return res;
}

char *key ()
{
	return ASTERISK_GPL_KEY;
}
